<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Class AbstractFromCollectionProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
abstract class AbstractFromCollectionProvider extends BaseProvider
{
    /**
     * @var int
     */
    protected $pageSize = 500;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * AbstractFromCollectionProvider constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    )
    {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @return bool|float|int|string|null
     */
    public function key()
    {
        return current($this->items)->getId();
    }

    /**
     * @return bool
     */
    public function valid()
    {
        if ($this->getCollection()->getCurPage() < $this->getCollection()->getLastPageNumber()) {
            if (current($this->items) !== false) {
                return true;
            }

            $this->handleItems($this->getCollection()
                ->clear()
                ->setCurPage($this->getCollection()->getCurPage() + 1)
                ->getItems());

            return true;
        }

        return current($this->items) !== false;
    }

    abstract public function getCollection();

    /**
     *
     */
    public function handleItems($items)
    {
        $this->items = $this->adapt($items);

        if (!$this->hasReadHandler()) {
            return;
        }

        $this->getReadHandler()->onRead($this->items, $this->pageSize, $this->getReadHandlerArguments());
    }

    /**
     * @param $items
     * @return mixed
     */
    abstract public function adapt($items);

    /**
     *
     */
    public function rewind()
    {
        $pageSize = (int)$this->scopeConfig->getValue('sync/data/index_bulk_size');
        $this->pageSize = ($pageSize) ? $pageSize : $this->pageSize;

        $this->handleItems($this->getCollection()
            ->clear()
            ->setPageSize($this->pageSize)
            ->setCurPage(1)
            ->getItems()
        );
    }
}
